﻿using System.Collections.ObjectModel;
using Android.Media;
using Com.Asreader.Ascameraxsdk;
using static AsCameraXMAUIDemo.CameraView;

namespace AsCameraXMAUIDemo;

public partial class BarcodePage : ContentPage
{
    private AsCameraXManager formManager;
    private AsCameraXCallback asCameraXCallback;
    private ObservableCollection<DataModel> dataModelList = new ObservableCollection<DataModel>();

    private Dictionary<string, DataModel> dataDic = new Dictionary<string, DataModel>();
    private bool listStatus;
    private bool continuousScanStatus;
    private DecodeThreed decodeThreed;
    ToneGenerator toneGenerator;
    public BarcodePage()
    {
        InitializeComponent();
        listStatus = true;
        continuousScanStatus = false;
        formManager = AsCameraXManager.sharedInstance(); 
        asCameraXCallback = AsCameraXCallback.sharedInstance();
        BindingContext = new CameraCropViewModel();
        try
        {
            string triggerMode = Preferences.Get("trigger_mode", "true");
            if (triggerMode == "true")
            {
                continuousScanStatus = true;
                ContinuousScan.Source = "trigger_mode_on.png";
                formManager.ScanMode(AsCameraXDecoder.ScanMode.ScanModeSingle);
            }
            else
            {
                continuousScanStatus = false;
                ContinuousScan.Source = "trigger_mode_off.png";
                formManager.ScanMode(AsCameraXDecoder.ScanMode.ScanModeContinuous);
            }
        }
        catch { }
        decodeThreed = new DecodeThreed(this);
        decodeThreed.Start();
        toneGenerator = new ToneGenerator(  Android.Media.Stream.Music , 100);
        dataModelList.CollectionChanged += (s, e) =>
        {
            Console.WriteLine("=====");
        };
    }

    protected override void OnAppearing()
    {
        base.OnAppearing();
        try
        {
            cameraView.WhenOnImageAvailableReceived += CameraView_WhenOnImageAvailableReceivedByte;
            asCameraXCallback.ReceivedDecodeImage = (AsCameraXResult asCameraXResult) =>
            {
                ReceivedDecodeImage(asCameraXResult);
            };
            asCameraXCallback.ReceivedMultipleDecodeImage = (IList<AsCameraXResult> list) =>
            {
                ReceivedMultipleDecodeImage(list);
            };
            asCameraXCallback.ReceivedDidBecomeActive = (bool status) =>
            {
                OnReceivedDidBecomeActive(status);
            };
            CodeLabel.Text = "";
            ListView.ItemsSource = dataModelList;
            if (cameraView.Camera != null)
            {
                _ = cameraView.StartCameraAsync();
            }
            formManager.StartScan();
        }
        catch { }
    }
    protected override void OnDisappearing()
    {
        base.OnDisappearing();
        try
        {
            formManager.StopScan();
            _ = cameraView.StopCameraAsync();
            asCameraXCallback.ReceivedDecodeImage = null;
            asCameraXCallback.ReceivedMultipleDecodeImage = null;
            asCameraXCallback.ReceivedDidBecomeActive = null;
        }
        catch { }
    }
    private void OnMoreClicked(object sender, EventArgs e)
    {
        Navigation.PushAsync(new SettingsPage());
    }
    public void ListButton_Clicked(object sender, EventArgs e)
    {
        if (listStatus == false)
        {
            listStatus = true;
            ListButton.Source = "playlist_plus";
            ListView.Opacity = 0;
        }
        else
        {
            listStatus = false;
            ListButton.Source = "playlist_remove";
            ListView.Opacity = 0.6;
        }
    }
    private void OnContinuousScanClicked(object sender, EventArgs e)
    {
        base.OnDisappearing();
        try
        {
            if (continuousScanStatus == false)
            {
                continuousScanStatus = true;
                ContinuousScan.Source = "trigger_mode_on.png";
                Preferences.Set("trigger_mode", "true");
                formManager.ScanMode(AsCameraXDecoder.ScanMode.ScanModeSingle);
                _ = cameraView.StopCameraAsync();
                formManager.StopScan();
                _ = cameraView.StartCameraAsync();
                formManager.StartScan();
            }
            else
            {
                continuousScanStatus = false;
                ContinuousScan.Source = "trigger_mode_off.png";
                Preferences.Set("trigger_mode", "false");
                formManager.ScanMode(AsCameraXDecoder.ScanMode.ScanModeContinuous);
                formManager.StartScan();
            }
        }
        catch { }
    }
    private void OnSingleClicked(object sender, EventArgs e)
    {
        try
        {
            string triggerMode = Preferences.Get("trigger_mode", "true");
            if (triggerMode == "true")
            {
                formManager.StartScan();
            }
        }
        catch { }
    }
    private void ReceivedDecodeImage(AsCameraXResult asCameraXResult)
    {
    }
    private void ReceivedMultipleDecodeImage(IList<AsCameraXResult> list)
    { 
        string triggerMode = Preferences.Get("trigger_mode", "true");
        if (list == null || (list != null && list.Count == 0)) {
            if (triggerMode == "true")
            {
                try
                {
                    MainThread.BeginInvokeOnMainThread(() =>
                    {
                        formManager.StartScan();
                    });
                }
                catch {
                }
            }
            return;
        }
        decodeThreed.setAsCameraXResult(list); 
    }
    private void ReceivedDecodeImage(IList<AsCameraXResult> list)
    {

        MainThread.BeginInvokeOnMainThread(() =>
        {
            CodeLabel.Text = "";
        });
        String resultStr = "";
        //((CameraCropViewModel)BindingContext).CurrentResultList = list;

        foreach (AsCameraXResult result1 in list)
        {
            string type = result1.SymbolType;
            string barcode = result1.SymbolText;
            resultStr = barcode + "\n" + type;
            DataModel dataChanged;
            if (dataDic.ContainsKey(type + barcode))
            {
                dataChanged = dataDic[type + barcode];
                dataChanged.Count++; 
                dataDic[type + barcode] = dataChanged;
            }
            else
            {
                dataChanged = new DataModel(type, barcode, 1); 
                dataDic.Add(type + barcode, dataChanged);
                dataModelList.Add(dataChanged);
            }
        }
        MainThread.BeginInvokeOnMainThread(() =>
        {
            CodeLabel.Text = resultStr;
            if (!listStatus)
            {
                ListView.ItemsSource = null;
                ListView.ItemsSource = dataModelList;
            }
        });
         PlayBeepSoundAndVibrate(); 
    }
    private void OnReceivedDidBecomeActive(bool status)
    {
        try
        {
            if (status)
            {
                _ = cameraView.StartCameraAsync();
                formManager.StartScan();
            }
            else
            {
                _ = cameraView.StopCameraAsync();
                formManager.StopScan();
            }
        }
        catch { }
    }
    private void PlayBeepSoundAndVibrate()
    {
        string beep = Preferences.Get("beep", "true");
        if (beep == "true")
        {
            toneGenerator.StartTone( Tone.CdmaNetworkCallwaiting);
        }
        string vibration = Preferences.Get("vibration", "true");
        if (vibration == "true")
        {
            Vibration.Default.Vibrate();
        }
    }
    private void CameraView_WhenOnImageAvailableReceivedByte(object sender, EventArgs e)
    {
        try
        {
            CustomEventArgs aaa = (CustomEventArgs)e;
            formManager.DecodeImage(aaa.data, aaa.width, aaa.height);

        }
        catch { }
    }

    private class DecodeThreed : Java.Lang.Thread
    {
        private IList<AsCameraXResult> currentAsCameraXResultList;
        BarcodePage currentBarcodePage;
        public DecodeThreed(BarcodePage barcodePage)
        {
            currentBarcodePage = barcodePage;
        }
        public override void Run()
        {
            base.Run();

            while (true)
            {
                if (currentAsCameraXResultList != null)
                {
                    currentBarcodePage.ReceivedDecodeImage(currentAsCameraXResultList);
                    currentAsCameraXResultList = null;
                }
            }
        }
        public void Log(string message)
        {
            string timeStamp = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss fff");
            string logEntry = $"{timeStamp} - {message}";
            Console.WriteLine(logEntry);
        }
        public void setAsCameraXResult(IList<AsCameraXResult> list)
        {
            currentAsCameraXResultList = list; 
        }
    }
}
