﻿using AsCameraXSDK;
using Com.Asreader.Ascameraxsdk;
using Com.Asreader.Aslicensesdk;
using Com.Asreader.Ascameraxsdk.Callback;
using Java.Lang;

namespace AsCameraXMAUIDemo
{
    public class AsCameraXManager
    {
        public class AsCameraXCertifiedCallback : Java.Lang.Object, IAsCameraXCertifiedCallback
        {
            AsCameraXCallback asCameraXCallback = AsCameraXCallback.sharedInstance();
            public AsCameraXCertifiedCallback()
            {

            }
            public void ReceivedCertifiedType(AsLicense asreader, AsLicenseCertifiedType certifiedType)
            {
                if (asCameraXCallback.ReceivedCertifiedType != null)
                {
                    asCameraXCallback.ReceivedCertifiedType(asreader, certifiedType);
                }
            }
        }
        public class AsCameraXDecodeImageCallback : Java.Lang.Object, IAsCameraXDecodeImageCallback
        {
            AsCameraXCallback asCameraXCallback = AsCameraXCallback.sharedInstance();
            public AsCameraXDecodeImageCallback()
            {

            }
            public void ReceivedDecodeImage(AsCameraXResult asCameraXResult)
            {
                if (asCameraXCallback.ReceivedDecodeImage != null)
                {
                    asCameraXCallback.ReceivedDecodeImage(asCameraXResult);
                }
            }
            public void ReceivedMultipleDecodeImage(IList<AsCameraXResult> list)
            {
                if (asCameraXCallback.ReceivedMultipleDecodeImage != null)
                {
                    asCameraXCallback.ReceivedMultipleDecodeImage(list);
                }
            }
        }
       
        static AsCameraXManager uniqueInstance;
        static readonly object locker = new object();
        internal int viewWidth = 720;
        internal int viewHeight = 1280;
        internal float viewScale = 1.0f; 

        public static AsCameraXManager sharedInstance()
        {
            if (uniqueInstance == null)
            {
                lock (locker)
                {
                    if (uniqueInstance == null)
                    {
                        uniqueInstance = new AsCameraXManager();
                    }
                }
            }
            return uniqueInstance;
        }
        public AsCameraXManager()
        {
            AsCameraXCertifiedCallback asCameraXCertifiedCallback = new AsCameraXCertifiedCallback();
            AsCameraXDecodeImageCallback asCameraXDecodeImageCallback = new AsCameraXDecodeImageCallback();
            AsCameraX.Instance.SetAsCameraXCertifiedCallback(asCameraXCertifiedCallback);
            AsCameraX.Instance.SetAsCameraXDecodeImageCallback(asCameraXDecodeImageCallback);
        }
        public void Init()
        {
            AsCameraX.Instance.SetContext(MainActivity.Instance);
            AsCameraX.Instance.FrequencyLimit = 0;
            AsCameraX.Instance.BarcodeRegionDisplay = true;
            AsCameraX.Instance.Charset = AsCameraXCharset.AsCameraXCharsetSHIFTJIS;
            SetAllBarcodeType();
        }
        public void ScanMode(AsCameraXDecoder.ScanMode status)
        {
            AsCameraX.Instance.SetScanMode(status);
        }
        public void StartScan()
        {
            AsCameraX.Instance.StartScan();
        }
        public void StopScan()
        {
            AsCameraX.Instance.StopScan();
        }
        public void DecodeImage(byte[] data, int width, int height)
        {
            if (data != null && data.Length > 0)
            {
                AsCameraX.Instance.DecodeImage(data, width, height);
            }
        }
        public string GetAPPVersion()
        {
            var activity = MainActivity.Instance;
            return activity.PackageManager.GetPackageInfo(activity.PackageName, 0).VersionName;
        }
        public string GetAPPBuild()
        {
            var activity = MainActivity.Instance;
            return activity.PackageManager.GetPackageInfo(activity.PackageName, 0).VersionCode.ToString();
        }
        public string GetAsBarcodeSDKVersion()
        {
            return AsCameraX.AsCameraXSDKVersion;
        }
        public string GetAsLicenseSDKVersion()
        {
            return AsLicense.SDKVersion;
        }
        public string GetAsBarcodeSDKDLL()
        {
            return AsCameraXSDKDLL.DLLVersion();
        }
        public string GetAsLicenseSDKDLL()
        {
            return AsCameraXSDKDLL.DLLVersion();
        }
        public bool GetCertifiedLicense()
        {
            return AsCameraX.Instance.CertifiedLicense;
        }
        private void SetAllBarcodeType()
        {
            List<Short> list = new List<Short>
            {
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeCODE39),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeUPCA),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeUPCE),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeEAN13),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeEAN8),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeI25),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeCODABAR),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeCODE128),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeGS1128),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypePDF),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeQRCODE),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeRSS14),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeRSSLIM),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeRSSEXP),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeCCCENABLE),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeCCABENABLE),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeUPCCOMPOSITE),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeTCIF),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeDEMOTERSS),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeEMULUCCEAN128),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeUQREN),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeDATABARLIMSECLEV),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeDATAMATRIX),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeMAXICODE),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeAZTEC),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeJAPANPOSTAL),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypePOSTNETUS),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeEANSUPPLEMENTAL),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeCODE39CHECKDIGITVERIFICATION),
                Short.ValueOf(AsCameraXSetBarcodeType.AsCameraXSetBarcodeTypeUPCAPREAMBLE),
            };
            AsCameraX.Instance.SymbologiesList = list;
        }
    }
}
