﻿namespace AsCameraXMAUIDemo
{
    public class ListCell : ViewCell
    {
        Label typeLabel, barcodeLabel, countLabel;

        public static readonly BindableProperty TypeProperty = BindableProperty.Create("Type", typeof(string), typeof(ListCell), "Type");
        public static readonly BindableProperty BarcodeProperty = BindableProperty.Create("Barcode", typeof(string), typeof(ListCell), "Barcode");
        public static readonly BindableProperty CountProperty = BindableProperty.Create("Count", typeof(int), typeof(ListCell), 0);

        public string Type
        {
            get { return (string)GetValue(TypeProperty); }
            set { SetValue(TypeProperty, value); }
        }

        public string Barcode
        {
            get { return (string)GetValue(BarcodeProperty); }
            set { SetValue(BarcodeProperty, value); }
        }

        public int Count
        {
            get { return (int)GetValue(CountProperty); }
            set { SetValue(CountProperty, value); }
        }

        public ListCell()
        {
            typeLabel = new Label { TextColor = Colors.Black, FontSize = 14, HorizontalOptions = LayoutOptions.Start };
            barcodeLabel = new Label { TextColor = Colors.Black, FontSize = 14, HorizontalOptions = LayoutOptions.Start };
            countLabel = new Label { TextColor = Colors.Black, FontSize = 12, HorizontalOptions = LayoutOptions.Start };

            StackLayout typeLayout = new StackLayout
            {
                Padding = new Thickness(15, 5, 0, 0),
                Orientation = StackOrientation.Horizontal,
                HorizontalOptions = LayoutOptions.StartAndExpand,
                Children = { typeLabel }
            };

            var viewLayout1 = new StackLayout()
            {
                Orientation = StackOrientation.Horizontal,
                Children = { typeLayout }
            };

            StackLayout barcodeLayout = new StackLayout
            {
                Padding = new Thickness(15, 0, 0, 0),
                Orientation = StackOrientation.Vertical,
                HorizontalOptions = LayoutOptions.StartAndExpand,
                Children = { barcodeLabel }
            };

            var viewLayout2 = new StackLayout()
            {
                Orientation = StackOrientation.Vertical,
                Children = { viewLayout1, barcodeLayout }
            };

            StackLayout countLayout = new StackLayout
            {
                Padding = new Thickness(0, 15, 15, 0),
                Orientation = StackOrientation.Vertical,
                HorizontalOptions = LayoutOptions.EndAndExpand,
                Children = { countLabel }
            };

            var viewLayout3 = new StackLayout()
            {
                Orientation = StackOrientation.Horizontal,
                Children = { viewLayout2, countLayout }
            };

            View = viewLayout3;
        }

        protected override void OnBindingContextChanged()
        {
            base.OnBindingContextChanged();
            if (BindingContext != null)
            {
                typeLabel.Text = Type;
                barcodeLabel.Text = Barcode;
                countLabel.Text = Count.ToString();
            }
        }
    }
}